<?php

namespace FiloBlu\Rma\Model;

use Exception;
use FiloBlu\Rma\Api\Data\EasyReturnInterface;
use FiloBlu\Rma\Helper\EasyReturnHelper;
use Magento\Framework\Data\Collection\AbstractDb;
use Magento\Framework\DataObject\IdentityInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Model\AbstractModel;
use Magento\Framework\Model\Context;
use Magento\Framework\Model\ResourceModel\AbstractResource;
use Magento\Framework\Pricing\PriceCurrencyInterface;
use Magento\Framework\Registry;
use Magento\Sales\Api\Data\OrderAddressInterface;
use Magento\Sales\Api\Data\OrderAddressInterfaceFactory;
use Magento\Sales\Api\OrderItemRepositoryInterface;
use Magento\Store\Api\StoreRepositoryInterface;
use Throwable;

/**
 *
 */
class EasyReturn extends AbstractModel implements IdentityInterface, EasyReturnInterface
{
    const CACHE_TAG = 'filoblu_rma_easyreturn';

    protected $_cacheTag = 'filoblu_rma_easyreturn';

    protected $_eventPrefix = 'filoblu_rma_easyreturn';

    protected $entityId;
    protected $customerFirstname;
    protected $customerLastname;
    protected $rmaEntityId;
    protected $street;
    protected $city;
    protected $regionId;
    protected $region;
    protected $postcode;
    protected $countryId;
    protected $theoricShippingAmount;

    /**
     * @var OrderAddressInterfaceFactory
     */
    private $addressInterfaceFactory;
    /**
     * @var EasyReturnHelper
     */
    private $easyReturnHelper;

    /**
     * @var OrderItemRepositoryInterface
     */
    private $orderItemRepository;
    /**
     * @var StoreRepositoryInterface
     */
    private $storeRepository;
    /**
     * @var PriceCurrencyInterface
     */
    private $priceCurrency;

    /**
     * EasyReturn constructor.
     * @param Context $context
     * @param Registry $registry
     * @param OrderAddressInterfaceFactory $addressInterfaceFactory
     * @param EasyReturnHelper $easyReturnHelper
     * @param OrderItemRepositoryInterface $orderItemRepository
     * @param StoreRepositoryInterface $storeRepository
     * @param PriceCurrencyInterface $priceCurrency
     * @param AbstractResource|null $resource
     * @param AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(
        Context                      $context,
        Registry                     $registry,
        OrderAddressInterfaceFactory $addressInterfaceFactory,
        EasyReturnHelper             $easyReturnHelper,
        OrderItemRepositoryInterface $orderItemRepository,
        StoreRepositoryInterface     $storeRepository,
        PriceCurrencyInterface       $priceCurrency,
        AbstractResource             $resource = null,
        AbstractDb                   $resourceCollection = null,
        array                        $data = [])
    {
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
        $this->addressInterfaceFactory = $addressInterfaceFactory;
        $this->easyReturnHelper = $easyReturnHelper;
        $this->orderItemRepository = $orderItemRepository;
        $this->storeRepository = $storeRepository;
        $this->priceCurrency = $priceCurrency;
    }

    /**
     * @return array|string[]
     */
    public function getIdentities()
    {
        return [self::CACHE_TAG . '_' . $this->getId()];
    }

    /**
     * @param string $firstname
     * @return $this
     */
    public function setFirstName(string $firstname)
    {
        return $this->setData(self::FIRSTNAME, $firstname);
    }

    /**
     * @param string $lastname
     * @return $this
     */
    public function setLastName(string $lastname)
    {
        return $this->setData(self::LASTNAME, $lastname);
    }

    /**
     * @return int
     */
    public function getRmaEntityId()
    {
        return $this->getData(self::RMA_ENTITY_ID);
    }

    /**
     * @param int $rmaEntityId
     * @return $this
     */
    public function setRmaEntityId(int $rmaEntityId)
    {
        return $this->setData(self::RMA_ENTITY_ID, $rmaEntityId);
    }

    /**
     * @param string $telephone
     * @return $this
     */
    public function setTelephone(string $telephone)
    {
        return $this->setData(self::TELEPHONE, $telephone);
    }

    /**
     * @return float
     */
    public function getTheoricShippingAmount()
    {
        return $this->getData(self::THEORIC_SHIPPING_AMOUNT);
    }

    /**
     * @param float $amount
     * @return $this
     */
    public function setTheoricShippingAmount(float $amount)
    {
        return $this->setData(self::THEORIC_SHIPPING_AMOUNT, $amount);
    }

    /**
     * @param array $data
     * @return $this
     * @throws LocalizedException
     */
    public function setEasyReturnData(array $data)
    {
        $this->setData($data);
        //save theoric shipping amount
        $theoreticShippingAmount = $this->easyReturnHelper->simulateEasyReturnShippingCost(
            $this->getPickUpAddress(),
            $data['store'],
            $data['weight']
        );
        $this->setTheoricShippingAmount($theoreticShippingAmount);
        return $this;
    }

    /**
     * @return OrderAddressInterface
     */
    public function getPickUpAddress()
    {
        $address = $this->addressInterfaceFactory->create();
        if ($this->getFirstName() === null) {
            return $address;
        }

        $address->setCountryId($this->getCountryId());
        $address->setCity($this->getCity())
            ->setRegionId($this->getRegionId())
            ->setPostcode($this->getPostcode())
            ->setFirstname($this->getFirstName())
            ->setLastname($this->getLastName())
            ->setStreet($this->getStreet())
            ->setTelephone($this->getTelephone());
        $address->setAddressType('shipping');

        if ($this->getRegion() != '') {
            $address->setRegion($this->getRegion());
        }

        return $address;
    }

    /**
     * @return string
     */
    public function getFirstName()
    {
        return $this->getData(self::FIRSTNAME);
    }

    /**
     * @return string
     */
    public function getCountryId()
    {
        return $this->getData(self::COUNTRY_ID);
    }

    /**
     * @param string $countryId
     * @return $this
     */
    public function setCountryId(string $countryId)
    {
        return $this->setData(self::COUNTRY_ID, $countryId);
    }

    /**
     * @return string
     */
    public function getCity()
    {
        return $this->getData(self::CITY);
    }

    /**
     * @param string $city
     * @return $this
     */
    public function setCity(string $city)
    {
        return $this->setData(self::CITY, $city);
    }

    /**
     * @return int
     */
    public function getRegionId()
    {
        return $this->getData(self::REGION_ID);
    }

    /**
     * @param int $regionId
     * @return $this
     */
    public function setRegionId(int $regionId)
    {
        return $this->setData(self::REGION_ID, $regionId);
    }

    /**
     * @return string
     */
    public function getPostcode()
    {
        return $this->getData(self::POSTCODE);
    }

    /**
     * @param string $postcode
     * @return $this
     */
    public function setPostcode(string $postcode)
    {
        return $this->setData(self::POSTCODE, $postcode);
    }

    /**
     * @return string
     */
    public function getLastName()
    {
        return $this->getData(self::LASTNAME);
    }

    /**
     * @return string
     */
    public function getStreet()
    {
        return $this->getData(self::STREET);
    }

    /**
     * @param string $street
     * @return $this
     */
    public function setStreet(string $street)
    {
        return $this->setData(self::STREET, $street);
    }

    /**
     * @return string
     */
    public function getTelephone()
    {
        return $this->getData(self::TELEPHONE);
    }

    /**
     * @return string
     */
    public function getRegion()
    {
        return $this->getData(self::REGION);
    }

    /**
     * @param string $region
     * @return $this
     */
    public function setRegion(string $region)
    {
        return $this->setData(self::REGION, $region);
    }

    /**
     * @param mixed $data
     * @return mixed
     */
    public function estimateReturnShippingCost($data)
    {
        $currency = '';

        try {

            $address = $this->addressInterfaceFactory->create();

            $address
                ->setCountryId($data['country_id'])
                ->setRegionId($data['region_id'])
                ->setCity($data['city'])
                ->setPostcode($data['postcode']);

            $weight = 0;
            $storeId = 0;
            foreach ($data['items'] as $item) {
                $product = $this->orderItemRepository->get($item['order_item_id']);

                $storeId = $product->getStoreId();
                if ($product->getWeight()) {
                    if ($item['qty_requested'] == '') {
                        $item['qty_requested'] = 1;
                    }
                    $weight += $product->getWeight() * $item['qty_requested'];
                }
            }
            $store = $this->storeRepository->getById($storeId);
            $currency = $this->priceCurrency->getCurrencySymbol($storeId);
            return $this->easyReturnHelper->simulateEasyReturnShippingCost($address, $store, $weight) . "$currency";
        } catch (Exception $e) {
            $this->_logger->critical($e->getMessage());
            return "$currency 0";
        } catch (Throwable $t) {
            $this->_logger->critical($t->getMessage());
            return "$currency 0";
        }

    }

    /**
     *
     */
    protected function _construct()
    {
        $this->_init(ResourceModel\EasyReturn::class);
    }
}
